;;;
;;; "sl-eval": The heart of the Small Lisp interpreter.
;;;
sl-eval[expr; fn-env; value-env] =
  [identifier-p[expr] --> apply-env[value-env; id-name[expr]];
   numeric-atom-expr-p[expr] --> numeric-val[expr];
   symbolic-atom-expr-p[expr] --> symbolic-val[expr];
   list-expr-p[expr] --> list-val[expr];
   fn-call-p[expr] -->
     sl-apply
       [callee[expr]; sl-evlis[arguments[expr]; fn-env; value-env];
        fn-env; reset-to-global-frame[value-env]];
   cond-expr-p[expr] --> sl-evcond[clauses[expr]; fn-env; value-env];
   let-expr-p[expr] -->
     sl-eval
       [final-expr[expr]; fn-env;
        extend-local-env[fn-env; value-env; local-defs[expr]]];
   otherwise --> error2[(Bad S-Lisp expression --); expr]]

;;;
;;; Evaluate a list of expressions in the context of function
;;; and value environments.
;;;
sl-evlis[explist; fn-env; val-env] =
  {val1 = sl-eval[first[explist]; fn-env; val-env] :
   [single-p[explist] --> list1[val1];
    otherwise -->
      cons[val1; sl-evlis[rest[explist]; fn-env; val-env]]]}

;;;
;;; Process conditional clauses by evaluating the clause
;;; predicates until one evaluates to T.  When this happens
;;; evaluate and return the corresponding result expression.
;;;
sl-evcond[condclauses; fn-env; env] =
  [sl-eval[predicate[first[condclauses]]; fn-env; env] -->
     sl-eval[result[first[condclauses]]; fn-env; env];
   single-p[condclauses] -->
     error[(Error in conditional expression -- no true predicate)];
   otherwise --> sl-evcond[rest[condclauses]; fn-env; env]]

;;;
;;; Update the local frame of the value environment with the
;;; bindings (local definitions) of a let expression.
;;;
extend-local-env[fn-env; value-env; local-defs] =
  {var-name = id-name[local-var[first[local-defs]]];
   val = sl-eval[local-val[first[local-defs]]; fn-env; value-env] :
   [single-p[local-defs] --> extend-env[value-env; var-name; val];
    otherwise -->
      extend-env
        [extend-local-env[fn-env; value-env; rest[local-defs]];
         var-name; val]]}
